/*******************************************************************************
* Project Name:      CORDIC_Rotation_Mode_Demo
* Version:           2.00
*
* Device Used:       PSoC 5LP CY8C5868AXI-LP035
* Software Used:     PSoC Creator v2.2
* Compiler Used:     ARM-RVDS Generic
* Related Hardware:  CY8CKIT-001 PSoC DVK
*********************************************************************************
* Description:
* - Pseudorandom angle value between 0 and 90 is generated and the CORDIC module
*   calculates the sine and cosine of that angle. The results are displayed on the
*   character LCD. When the calculation is complete, LED1 turns on.
* 
* LCD Display:
* - Ang: Displays the desired angle value.
* - Sin: Displays the sine value. (This will almost always be negative due to the 
*        way CORDIC functions. It may be positive if the angle value is near 0).
* - Cos: Displays the cosine value. (This value will always be positive).
*
* Note:
* - The values obtained from CORDIC are rescaled for easy viewing on the LCD. 
* - The results may exceed 1 if the angle values are near either 0 or 90.
* - The signs of the sine and cosine results need to be allocated post-CORDIC
*   calculation since it only accepts angle values from between 0 and 90.
*
********************************************************************************
* Copyright (2012), Cypress Semiconductor Corporation.
********************************************************************************
* This software, including source code, documentation and related materials
* ("Software") is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and 
* foreign), United States copyright laws and international treaty provisions. 
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the 
* Cypress source code and derivative works for the sole purpose of creating 
* custom software in support of licensee product, such licensee product to be
* used only in conjunction with Cypress's integrated circuit as specified in the
* applicable agreement. Any reproduction, modification, translation, compilation,
* or representation of this Software except as specified above is prohibited 
* without the express written permission of Cypress.
* 
* Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, 
* EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes to the Software without notice. 
* Cypress does not assume any liability arising out of the application or use
* of Software or any product or circuit described in the Software. Cypress does
* not authorize its products for use as critical components in any products 
* where a malfunction or failure may reasonably be expected to result in 
* significant injury or death ("High Risk Product"). By including Cypress's 
* product in a High Risk Product, the manufacturer of such system or application
* assumes all risk of such use and in doing so indemnifies Cypress against all
* liability. Use of this Software may be limited by and subject to the applicable
* Cypress software license agreement.
*******************************************************************************/

#include <device.h>
#include <stdlib.h>

CY_ISR_PROTO(ISR_OUT);
void Num_Print(uint8 data_in, uint8 i, uint8 k);
void Sample_Print(int32 data_in, uint8 i, uint8 k);

uint8 output_rdy;
		
/*******************************************************************************
* Function Name: CY_ISR(ISR_OUT)
********************************************************************************
* Summary:
*  Interrupt handler for the interrupt from the CORDIC component. This is used
*  to retrieve the results from the CORDIC output registers.
*
* Parameters:
*  None.
* 
* Return: 
*  None.
*
*******************************************************************************/
CY_ISR(ISR_OUT)
{   
	output_rdy = 1;
}

void main()
{	
    int16 sin_result;
	int16 cos_result;
	int16 sin_actual;
	int16 cos_actual;
	uint16 desired_angle;
	
	CYGlobalIntEnable;	
	
	LCD_Char_1_Start();
    
	/* ISR set up */
	isr_1_Start();
    isr_1_SetVector(ISR_OUT);
	
	LCD_Char_1_Position(0, 0);
    LCD_Char_1_PrintString("Ang|");
	LCD_Char_1_Position(1, 3);
    LCD_Char_1_PrintString("|");
	LCD_Char_1_Position(0, 5);
    LCD_Char_1_PrintString("Sin");
	LCD_Char_1_Position(1, 5);
    LCD_Char_1_PrintString("Cos");
	
	desired_angle = (uint16)(rand()%0x5A);
	RFMS_CORDIC_1_Start(desired_angle<<8,0,0);
	Num_Print(desired_angle,1,0);
	
	for(;;)
    {   
		/* Polling the CORDIC output */
        if ( output_rdy == 1)
        {
		    output_rdy = 0;
			RFMS_CORDIC_1_Stop();
			Pin_1_Write(1);
			
			RFMS_CORDIC_1_GetData(&sin_result,&cos_result);
			
			/* Desired magnitude = 155  */
			/* Normalizer = 0.607252935 */
			/* result*(Normalizer/Desired Magnitude) */
			sin_actual = (int16)(10000*(float)(sin_result*0.00391776)); 
			cos_actual = (int16)(10000*(float)(cos_result*0.00391776));
			
			Sample_Print(sin_actual, 0, 9);
			Sample_Print(cos_actual, 1, 9);
		
			CyDelay(1000); /* 1 second delay */
			
			Pin_1_Write(0);
			
			desired_angle = (uint16)(rand()%0x5A);
			
			RFMS_CORDIC_1_Start(desired_angle<<8,0,0);
			Num_Print(desired_angle,1,0);
        }
    }
}

/********************************************************************************
* Function Name: Num_Print
*******************************************************************************
* Summary:
*  Prints the data_in on the LCD in decimal format.
*
* Parameters:
*	uint8 data_in, uint8 i, uint8 k
*
* Return:
*	void
*
*******************************************************************************/
void Num_Print(uint8 data_in, uint8 i, uint8 k)
{
    uint8 c;
    char8 number[2u];
    
    /* Procedure of number conversion to decimal system */
    c = 1u;
    number[0u] = 0x30u;
    number[1u] = 0x30u;
    number[2u] = '\0';
    while (data_in >= 10u)
    {
        number[c--] = (data_in % 10u) + 0x30u;
        data_in /= 10u;
    }            
    number[c] = (data_in % 10u) + 0x30u;
    
	LCD_Char_1_Position(i, k);
    LCD_Char_1_PrintString(number);
}

/********************************************************************************
* Function Name: Sample_Print
*******************************************************************************
* Summary:
*  Prints the data_in on the LCD in decimal format.
*
* Parameters:
*	int32 data_in, uint8 i, uint8 k
*
* Return:
*	void
*
*******************************************************************************/
void Sample_Print(int32 data_in, uint8 i, uint8 k)
{
    uint8 c;
	char8 integer[1u];
    char8 fraction[5u];
	char8 sign[1u];
	int32 tmp;
    
	if (data_in < 0)
	{
		data_in = 0 - data_in; //negative to positive for printing
		strcpy(sign, "-");
	}
	else
		strcpy(sign, " ");
	
	/* Calculate the integer part */
	tmp = data_in / 10000;
    integer[0u] = 0x30u;
    integer[0u] = (tmp % 10u) + 0x30u;
	
	/* Calculate the fractional part */
    c = 3u;
    fraction[0u] = 0x30u;
    fraction[1u] = 0x30u;
    fraction[2u] = 0x30u;
    fraction[3u] = 0x30u;
    fraction[4u] = '\0';
    while (data_in >= 10u)
    {
        fraction[c--] = (data_in % 10u) + 0x30u;
        data_in /= 10u;
    }            
    fraction[c] = (data_in % 10u) + 0x30u;
	
	LCD_Char_1_Position(i, k);
    LCD_Char_1_PrintString(sign);
	LCD_Char_1_Position(i, k+1);
    LCD_Char_1_PrintString(integer);
	LCD_Char_1_Position(i, k+2);
    LCD_Char_1_PrintString(".");
    LCD_Char_1_Position(i, k+3);
    LCD_Char_1_PrintString(fraction);
}

/* [] END OF FILE */
