/*******************************************************************************
* Project Name:      CORDIC_Vectoring_Mode_Demo
* Version:           2.00
*
* Device Used:       PSoC 5LP CY8C5868AXI-LP035
* Software Used:     PSoC Creator v2.2
* Compiler Used:     ARM-RVDS Generic
* Related Hardware:  CY8CKIT-001 PSoC DVK
*********************************************************************************
* Description:
* - Pseudorandom angle value between 0 and 90 is generated and it's corresponding
*   cosine and sine values with fixed magnitude are fed in to the RFMS_CORDIC module. 
*   The component calculates the arctan2 of the vectors and the hypotenuse. These
*   results are retrieved and displayed on the character LCD. When the calculation 
*   is complete, LED1 turns on.
* 
* LCD Display:
* - x: Displays the cosine of the generated angle normalized to 1.
* - y: Displays the sine of the generated angle normalized to 1.
* - A: The arctan2 result shown in degrees.
* - H: The hypotenuse result normalized to 1.
*
* Note:
* - The values obtained from RFMS_CORDIC are scaled when sending to the module
* - and are rescaled for easy viewing on the LCD.
* - The CORDIC algorithm works between 0 and 90 degrees. Therefore signs need to
* - be managed externally when dealing with angles larger than 90.
*
********************************************************************************
* Copyright (2012), Cypress Semiconductor Corporation.
********************************************************************************
* This software, including source code, documentation and related materials
* ("Software") is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and 
* foreign), United States copyright laws and international treaty provisions. 
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the 
* Cypress source code and derivative works for the sole purpose of creating 
* custom software in support of licensee product, such licensee product to be
* used only in conjunction with Cypress's integrated circuit as specified in the
* applicable agreement. Any reproduction, modification, translation, compilation,
* or representation of this Software except as specified above is prohibited 
* without the express written permission of Cypress.
* 
* Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, 
* EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes to the Software without notice. 
* Cypress does not assume any liability arising out of the application or use
* of Software or any product or circuit described in the Software. Cypress does
* not authorize its products for use as critical components in any products 
* where a malfunction or failure may reasonably be expected to result in 
* significant injury or death ("High Risk Product"). By including Cypress's 
* product in a High Risk Product, the manufacturer of such system or application
* assumes all risk of such use and in doing so indemnifies Cypress against all
* liability. Use of this Software may be limited by and subject to the applicable
* Cypress software license agreement.
*******************************************************************************/

#include <device.h>
#include <stdlib.h>
#include <math.h>

#define PI 3.14159265

CY_ISR_PROTO(ISR_OUT);
void Sample_Print_2(int32 data_in, uint8 i, uint8 k);
void Sample_Print(int32 data_in, uint8 i, uint8 k);

uint8 output_rdy;
		
/*******************************************************************************
* Function Name: CY_ISR(ISR_OUT)
********************************************************************************
* Summary:
*  Interrupt handler for the interrupt from the CORDIC component. This is used
*  to retrieve the results from the CORDIC output registers.
*
* Parameters:
*  None.
* 
* Return: 
*  None.
*
*******************************************************************************/
CY_ISR(ISR_OUT)
{   
	output_rdy = 1;
}

/*******************************************************************************
* Function Name: main
********************************************************************************
* Summary:
*  Main program
*
* Parameters:
*  None.
* 
* Return: 
*  None.
*
*******************************************************************************/
void main()
{	
    int16 arc_result;
	int16 hyp_result;
	int16 arc_norm;
	int16 hyp_norm;
	uint16 desired_angle;
	
	uint16 x,y;
	int16 x_actual,y_actual;
	
	CYGlobalIntEnable;	
	
	LCD_Char_1_Start();
    
	/* ISR set up */
	isr_1_Start();
    isr_1_SetVector(ISR_OUT);
	
	LCD_Char_1_Position(0, 0);
    LCD_Char_1_PrintString("x");
	LCD_Char_1_Position(1, 0);
    LCD_Char_1_PrintString("y");
	LCD_Char_1_Position(0, 8);
    LCD_Char_1_PrintString("|A");
	LCD_Char_1_Position(1, 8);
    LCD_Char_1_PrintString("|H");
	
	/* Calculate the cos and sin of a pseudorandom angle */
	/* The magnitude is fixed to 77 for maximum resolution */
	desired_angle = (uint16)(rand()%0x5A);
	x = (uint16)(77*256*(cos((float)(desired_angle*PI/180))));
	y = (uint16)(77*256*(sin((float)(desired_angle*PI/180))));
	
	/* Scale for printing */
	x_actual = (int16)(1000*(float)(x*(5.07305195e-5))); /* (1/77) for printing */
	y_actual = (int16)(1000*(float)(y*(5.07305195e-5)));
	
	RFMS_CORDIC_1_Start(0,x,y);
	Sample_Print(x_actual,0,1);
	Sample_Print(y_actual,1,1);
	
	for(;;)
    {   
		/* Polling the CORDIC output */
        if ( output_rdy == 1)
        {
		    output_rdy = 0;
			RFMS_CORDIC_1_Stop();
			Pin_1_Write(1);
			
			RFMS_CORDIC_1_GetData(&arc_result,&hyp_result);
			 
			/* Desired magnitude = 77  */
			/* Factor to normalize to 1 = 256  */
			/* Final Magnitude = 77*256 = 19712 */
			/* Normalizer = 0.607252935 */
			/* result*(Normalizer/Final Magnitude) */
			hyp_norm = (int16)(10000*(float)(hyp_result*(3.08062568e-5)));
			arc_norm = (int16)(100*(float)(arc_result*0.00390625)); /* (1/256) for printing */
			
			Sample_Print_2(arc_norm, 0, 10);
			Sample_Print(hyp_norm, 1, 10);
			
			CyDelay(1000); /* 1 second delay */
			
			Pin_1_Write(0);
			
			/* Calculate the cos and sin of a random angle */
			/* The magnitude is fixed to 77 for maximum resolution */
			desired_angle = (uint16)(rand()%0x5A);
			x = (uint16)(77*256*(cos((float)(desired_angle*PI/180))));
			y = (uint16)(77*256*(sin((float)(desired_angle*PI/180))));
			
			/* Scale for printing */
			x_actual = (int16)(1000*(float)(x*(5.07305195e-5))); /* (1/77) for printing */
			y_actual = (int16)(1000*(float)(y*(5.07305195e-5)));
			
			RFMS_CORDIC_1_Start(0,x,y);
			Sample_Print(x_actual,0,1);
			Sample_Print(y_actual,1,1);
        }
    }
}

/********************************************************************************
* Function Name: Sample_Print_2
*******************************************************************************
* Summary:
*  Prints data_in on the LCD as 2 integer precision and 2 decimal points.
*
* Parameters:
*	int32 data_in, uint8 i, uint8 k
*
* Return:
*	void
*
*******************************************************************************/
void Sample_Print_2(int32 data_in, uint8 i, uint8 k)
{
    uint8 c;
	char8 integer[2u];
    char8 fraction[3u];
	char8 sign[1u];
	int32 tmp;
    
	if (data_in < 0)
	{
		data_in = 0 - data_in; //negative to positive for printing
		strcpy(sign, "-");
	}
	else
		strcpy(sign, " ");
	
	/* Calculate the integer part */
	c = 1u;
	tmp = data_in / 100;
    integer[0u] = 0x30u;
	integer[1u] = 0x30u;  
    integer[1u] = (tmp % 10u) + 0x30u;
	tmp /= 10u;
	integer[0u] = (tmp % 10u) + 0x30u;
	
	/* Calculate the fractional part */
    c = 1u;
    fraction[0u] = 0x30u;
    fraction[1u] = 0x30u;
    fraction[2u] = '\0';
    while (data_in >= 10u)
    {
        fraction[c--] = (data_in % 10u) + 0x30u;
        data_in /= 10u;
    }            
    fraction[c] = (data_in % 10u) + 0x30u;
	
	LCD_Char_1_Position(i, k);
    LCD_Char_1_PrintString(sign);
	LCD_Char_1_Position(i, k+1);
    LCD_Char_1_PrintString(integer);
	LCD_Char_1_Position(i, k+3);
    LCD_Char_1_PrintString(".");
    LCD_Char_1_Position(i, k+4);
    LCD_Char_1_PrintString(fraction);
}

/********************************************************************************
* Function Name: Sample_Print
*******************************************************************************
* Summary:
*  Prints data_in on the LCD with 1 integer precision and 3 decimal points
*
* Parameters:
*	int32 data_in, uint8 i, uint8 k
*
* Return:
*	void
*
*******************************************************************************/
void Sample_Print(int32 data_in, uint8 i, uint8 k)
{
    uint8 c;
	char8 integer[2u];
    char8 fraction[4u];
	char8 sign[1u];
	int32 tmp;
    
	if (data_in < 0)
	{
		data_in = 0 - data_in; //negative to positive for printing
		strcpy(sign, "-");
	}
	else
		strcpy(sign, " ");
	
	/* Calculate the integer part */
	tmp = data_in / 10000;
    integer[0u] = 0x30u;
    integer[0u] = (tmp % 10u) + 0x30u;
	
	/* Calculate the fractional part */
    c = 2u;
    fraction[0u] = 0x30u;
    fraction[1u] = 0x30u;
    fraction[2u] = 0x30u;
    fraction[3u] = '\0';
    while (data_in >= 10u)
    {
        fraction[c--] = (data_in % 10u) + 0x30u;
        data_in /= 10u;
    }            
    fraction[c] = (data_in % 10u) + 0x30u;
	
	LCD_Char_1_Position(i, k);
    LCD_Char_1_PrintString(sign);
	LCD_Char_1_Position(i, k+1);
    LCD_Char_1_PrintString(integer);
	LCD_Char_1_Position(i, k+2);
    LCD_Char_1_PrintString(".");
    LCD_Char_1_Position(i, k+3);
    LCD_Char_1_PrintString(fraction);
    LCD_Char_1_Position(i, k+6);
	LCD_Char_1_PrintString(" ");
}

/* [] END OF FILE */
